#!/usr/bin/perl
#
# smst.pl: By Tomasz Sterna <tomek@smoczy.net> @2003
# License: GPL v.2.x
# (Obtain full license text from http://www.fsf.org/)
#
# Release: 4
#
# ChangeLog:
# 4: setting own signature
#    jabber:iq:browse NS support (Jajcus)
#    jabber:iq:version NS support
# 3: 1stWap gateway support (you have to register at http://www.1rstwap.com/ )
#    jabber:iq:register NS finished
#    
# 2: jabber:iq:gateway NS added
#    jabber:iq:register NS semi-finished
 
				  
# BEGIN CONFIGURATION
use constant SERVICE_NAME => 'sms.localhost';   # full name of the SMS transport.
use constant SERVER       => '127.0.0.1';       # ip/hostname of the jabber server to connect to
use constant PORT         => 7010;              # port on the jabber server to connect to
use constant SECRET       => 'secret';          # SECRET phrase for jabber component connection.
use constant PASSWORDS    => './pass';          # spool file for 1stWap gateway passwords
# END BASIC CONFIGURATION

# BEGIN EXTRA CONFIGURATION
use constant VERBOSE      => 2;                 # Verbosity level for logging output
# END EXTRA CONFIGURATION

#
use constant RELEASE      => 4;

use Net::Jabber qw(Component);
use LWP::UserAgent;
use HTTP::Request::Common qw(GET POST);
use HTTP::Response;
use SMS;
use strict;

$SIG{KILL} = \&Stop;
$SIG{TERM} = \&Stop;
$SIG{INT} = \&Stop;

# SMS Gateway Associations
use constant DEFAULT_GATEWAY => '1stwap';
my %sms_prefixes = (
			"1stwap"	=> "",
			"era"		=> "48600 48602 48604 48606 48608 48692 48694 48696 48698",
			"plus"		=> "48601 48603 48605 48607 48609 48691 48693 48695 48697 48699"
		);
my %sms_callbacks = (
			"1stwap"	=> \&Send1wap,
			"era"		=> \&SendLIB,
			"plus"		=> \&SendLIB,
		);

my $Connection = new Net::Jabber::Component(debuglevel=>0, debugfile=>"stdout");

my $status = $Connection->Connect("hostname" => SERVER, "port" => PORT, "secret" => SECRET, "componentname" => SERVICE_NAME);

if (!(defined($status))) {
  log1( "ERROR:  Jabber server is down or connection was not allowed.");
  log1( "        ($!)\n");
  exit(0);
}

$Connection->SetCallBacks("message" => \&InMessage, "presence" => \&InPresence, "iq" => \&InIQ);

$Connection->Connect();
log2("Connected to " . SERVER . "." . PORT . "...\n");

# Loop until we're finished.
while (true) {
  Stop() unless defined($Connection->Process(1));
}

log1("ERROR: The connection was killed...\n");

exit(0);

sub InMessage{
  # Incoming message. Let's try to send it via SMS.
  # If error we've got, we log it... ;-)

	my $sid = shift;
	my $message = shift;
	my $from = $message->GetFrom();
	my $to = $message->GetTo();
	my $body = $message->GetBody();
	my $type = $message->GetType();
	my ($number) = split(/@/, $to);
	if ($type eq 'error') {
		log1("Error received: \n" . $message->GetXML());
		return;
	}
  	if ( $number !~ /^\+[0-9]{11,11}$/ ) {
		log3("Invalid number `$number' got.");
		sendError($message, $from, $to, 404, "Invalid number");
		return;
	}

	if ( $body eq "" ) {
		log3("Dropping empty message from `$from' to number `$number'.");
		return;
	}

	my $gateway = $to;
	if ( $gateway =~ /\// ) {
		$gateway =~ s!^.*\@.*\/(.*)$!\1!;
	}
	else {
		$gateway = DEFAULT_GATEWAY;
		my $prefix = substr($number, 1, 5);
		foreach my $gwname ( keys %sms_prefixes ){
			if (  $sms_prefixes{$gwname} =~ /$prefix/ ) {
				$gateway = $gwname;
			}
		}
	}
	
	log3("Message from `$from' through gateway `$gateway' for number `$number'.");
	
	if ($gateway) {
		my ($barejid) = split(/\//, $from);
		my $ret = &{$sms_callbacks{$gateway}}($number, $barejid, $body);
		$message->SetTo("$from");
		$message->SetBody($ret);
		$message->SetFrom("$to");
		$Connection->Send($message);
	}
	else {
		sendError($message, $from, $to, 404, "No proper gateway");
	}
}

sub SendLIB {
	my ($number, $from, $msg) = @_;
	$number = substr($number, 3, 9);
	log3("SendLIB: sending message to number `$number'.");
	
	my $user = getUserPass($from, '1stwap');

	my $sms = new SMS ( numer => $number );

	if ( $user->{signature} ne '' ) {
		$sms->from($user->{signature});
	} else {
		$sms->from($from);
	}
	$sms->message($msg);
	if ($sms->send) {
		log3("Success");
		return $sms->response;
	}
	else {
		log3("Failed");
		return "Sending failed.";
	}
}

sub getUserPass {
	my ($from, $gateway) = @_;
	my ($barejid) = split (/\//, $from);
	my $passfile = PASSWORDS."/$barejid";

	my $user = {};

	open(F, "<$passfile");
	seek(F, 0, 0);
	local $/ = "\n";

	while (<F>) {
		chop;
		($user->{gateway}, $user->{name}, $user->{password}, $user->{signature}) = split(':');
		log3('Got password: '.$user->{name}.'/'.$user->{password}.'@'.$user->{gateway}." for `$barejid'.");
		last if ($user->{gateway} eq $gateway);
	}
	$user->{name} = '' if ( ! $user->{name} );
	$user->{password} = '' if ( ! $user->{password} );
	$user->{signature} = '' if ( ! $user->{signature} );
	if ($user->{gateway} ne $gateway) {
		$user->{gateway} = $gateway;
	}

	return $user;
}

sub Send1wap {
	my ($number, $from, $msg) = @_;
	$number = substr($number, 1, 11);
	log3("Send1wap: sending message to number `$number'.");
	
	my $user = getUserPass($from, '1stwap');

	return "You need to register to use 1stWAP gateway." if ($user->{name} eq '');
	
	my $result = send_firstwap($number, $msg, $user->{name}, $user->{password});
	if ($result) {
		log3("Success");
		return $result;
	}
	else {
		log3("Failed");
		return "Sending failed.";
	}
}

sub send_firstwap {
    my $number = shift;
    my $mess = shift;
    my $login = shift;
    my $password = shift;

    my $ua = new LWP::UserAgent;
    $ua->agent("Mozilla/3.0 (X11; I; Linux 2.2.9 i686)");
    $ua->timeout(900);

    # Pobieramy sessionID
    my $get = GET "http://www.1rstwap.com/partners/go.to/1rstwap";
    my $res = $ua->request($get);
    my $sessionid = $res->headers->as_string('');
    $sessionid =~ s/^.*PHPSESSID=([0-9a-f]{40,40});.*$/\1/i;

    # Logujemy sie
    my $post = 
	POST "http://www.1rstwap.com/partners/go.php?PHPSESSID=$sessionid&sestime=01112001140708&pid=login",
	[UserID => $login,
	 Passwd => $password,
	 submitButtonName => 'Sign In',
	 language => '/partners/go.php?PHP_SESSID=&pid=first&LangID=en&cobid=1RSTWAP',
	 cobid => '1RSTWAP' ],
	Cookie => "PHPSESSID=$sessionid",
	Referer => "http://www.1rstwap.com/partners/go.php?PHPSESSID=&pid=first&LangID=en";

    $res = $ua->request($post);

    # Pobieramy Formularz
    $get = GET "http://www.1rstwap.com/partners/go.php?pid=addressbook&aid=smswindow&step=openfmenu",
        Cookie => "PHPSESSID=$sessionid",
	Referer => "http://www.1rstwap.com/partners/go.php?PHPSESSID=$sessionid&sestime=01112001140708&pid=login";
    $res = $ua->request($get);
    my @source = split (/\n/, $res->content);
    my $request = (grep /\<form/, @source)[0];
    $request =~ s/.*form\s+name="frmnewsmsuser"\s+action="([^"]*).*/\1/;
    my $phone = (grep /\<input.*SENDER/, @source)[0];
    $phone =~ s/.*name="SENDER"\s+value="([0-9]*)".*/\1/;

    # i w koncu wysylamy
    my $post = 
	POST 'http://www.1rstwap.com'.$request,
	[hp => $number,
	 m => $mess,
	 submitButtonName => 'Send',
	 SENDER => $phone,
	 COUNTER => 160 - length($mess) ],
	Cookie => "PHPSESSID=$sessionid",
	Referer => "http://www.1rstwap.com/partners/go.php?pid=addressbook&aid=smswindow&step=openfmenu";

    $res = $ua->request($post);
    my $result = grep /has\s+been\s+sent/, split (/\n/, $res->content);
    if ($result) {
    	# pobierzmy ilosc wiadomosci pozostalych do wyslania
	$get = GET "http://www.1rstwap.com/partners/go.php?pid=addressbook",
		Cookie => "PHPSESSID=$sessionid";
	$res = $ua->request($get);
	@source = split (/\n/, $res->content);
	$result = (grep /possible.*messages/, @source)[0];
	$result =~ /.*.b.([0-9]+).\/b.\s+of\s+possible\s+.b.([0-9]+).\/b.\s+messages\s+sent\s+this\s+week.\s+.b.([0-9]+).\/b.\s+messages\s+left.*/;
	return "Wysłany. ($1 z $2/tydzien. Pozostało: $3)";
    }
    else {
        return 0;
    }
}

sub InIQ{
  # Incoming IQ. Handle jabber:iq:registration with add/remove source dialog, return error 501 for other NS's.
	my $sid = shift;
	my $iq = shift;

	my $from = $iq->GetFrom();
	my $to = $iq->GetTo();
	my $id = $iq->GetID();
	my $type = $iq->GetType();
	my $query = $iq->GetQuery();
	return unless $query;
	if ($type eq 'error') {
		log1("Error received: \n" . $iq->GetXML());
		return;
	}
	my $xmlns = $query->GetXMLNS();
	if ($xmlns eq 'jabber:iq:register') {
		if ($type eq 'get') {
			$iq->SetType('result');
			$iq->SetFrom($iq->GetTo());
			$iq->SetTo($from);
			$query->SetInstructions('Send a message to: +48TELNUMBER@'.SERVICE_NAME." to send an SMS.\n".
				"You can add a contact like this to your roster.\n\n".
				"If You give your Nick, it will be used as your signature.\n\n".
				'Give your Username and Password if You want to use 1stWap gateway.');
			my $user = getUserPass($from, '1stwap');
			$query->SetNick($user->{signature});
			$query->SetUsername($user->{name});
			$query->SetPassword('');
			$Connection->Send($iq);
		}
		elsif ($type eq 'set') {
			my $gateway = '1stwap'; # TODO! ->GetName() but only one gateway with passwords so far
			my $name = $query->GetUsername();;
			my $pass = $query->GetPassword();
			my $signature = $query->GetNick();
			my ($barefrom) = split (/\//, $from);

			my $passfile = PASSWORDS."/$barefrom";
			open(F, ">$passfile") or die "Couldn't open `$passfile' for writing";
			print(F "$gateway:$name:$pass:$signature\n");

			$iq->SetType('result');
			$iq->SetFrom($iq->GetTo());
			$iq->SetTo($from);
			$Connection->Send($iq);
			my $presence = new Net::Jabber::Presence();
			sendPresence($presence, $from, SERVICE_NAME.'/registered', 'subscribe');
		}
		else {
			sendError($iq, $from, $to, 501, 'Not Implemented');
		}
	}
	elsif ($xmlns eq 'jabber:iq:gateway') {
		if ($type eq 'get') {
			$iq->SetType('result');
			$iq->SetFrom($iq->GetTo());
			$iq->SetTo($from);
			$query->SetDesc('Prosze podac nr telefonu na ktory chcesz wysylac SMS.');
			$query->SetPrompt('Numer Telefonu');
			$Connection->Send($iq);
		}
		elsif ($type eq 'set') {
			my $number = $query->GetPrompt();
			if ( $number =~ /[0-9]{11,11}/ ) {
				$number =~ s/^.*([0-9]{11,11}).*$/\1/;
			}
			elsif ( $number =~ /[0-9]{9,9}/ ) {
				$number =~ s/^.*([0-9]{9,9}).*$/48\1/;
			}
			else {
				sendError($iq, $from, $to, 406, 'Not Acceptable');
			}
			$iq->SetType('result');
			$iq->SetFrom($iq->GetTo());
			$iq->SetTo($from);
			$query->SetPrompt("+$number@".SERVICE_NAME);
			$query->SetJID("+$number@".SERVICE_NAME);
			$Connection->Send($iq);
		}
		else {
			sendError($iq, $from, $to, 501, 'Not Implemented');
		}
	}
	elsif ($xmlns eq 'jabber:iq:browse') {
		my $namespaces = [ 'jabber:iq:register', 'jabber:iq:gateway' ];
		if ($type eq 'get') {
			$iq->SetType('result');
			$iq->SetFrom($iq->GetTo());
			$iq->SetTo($from);
			$query->SetJID(SERVICE_NAME);
			$query->SetCategory("service");
			$query->SetType("sms");
			$query->SetName("SMS Transport");
			$query->SetNS($namespaces);
			$Connection->Send($iq);
		}
		else {
			sendError($iq, $from, $to, 501, 'Not Implemented');
		}
	}
	elsif ($xmlns eq 'jabber:iq:version') {
		if ($type eq 'get') {
			$iq->SetType('result');
			$iq->SetFrom($iq->GetTo());
			$iq->SetTo($from);
			$query->SetName("SMS Transport");
			$query->SetVer('R'.RELEASE);
			$query->SetOS($^O);
			$Connection->Send($iq);
		}
		else {
			sendError($iq, $from, $to, 501, 'Not Implemented');
		}
	}
	else {
		sendError($iq, $from, $to, 501, 'Not Implemented');
	}
}

sub sendError {
	my ($msg, $to, $from, $code, $text) = @_;
	
	$msg->SetType('error');
	$msg->SetFrom($from);
	$msg->SetTo($to);
	$msg->SetErrorCode($code);
	$msg->SetError($text);
	$Connection->Send($msg);
}

sub sendPresence {
	my ($pres, $to, $from, $type, $show, $status) = @_;

	$pres->SetType($type);
	$pres->SetShow($show);
	$pres->SetStatus($status);
	$pres->SetTo($to);
	$pres->SetFrom($from);
	log3("Sending presence `$type' from $from to $to.");
	$Connection->Send($pres);
}

# send presences for given number with resources indicating gateways
sub sendGWNumPresences {
	my ($number, $to) = @_;	
	my $prefix = substr($number, 1, 5);
	my $presence = new Net::Jabber::Presence();

	$presence->SetType('available');
	$presence->SetShow(undef);
	$presence->SetStatus(undef);
	$presence->SetTo($to);
	foreach my $gwname ( keys %sms_prefixes ){
		if (  $sms_prefixes{$gwname} =~ /$prefix/ ) {
			$presence->SetFrom($number."@".SERVICE_NAME."/".$gwname);
			$presence->SetPriority(5);
			log3("Sending presence from ".$presence->GetFrom()." to $to.");
			$Connection->Send($presence);
		}
	}
	my $user = getUserPass($to, '1stwap');
	if (($user->{name} ne '') && ($user->{password} ne '')) {
		$presence->SetFrom($number."@".SERVICE_NAME.'/1stwap');
		$presence->SetPriority(3);
		log3("Sending presence from ".$presence->GetFrom()." to $to.");
		$Connection->Send($presence);
	}
}

sub InPresence {
  # Incoming presence. Reply to subscription requests with proper subscription actions, reply to probe presence with gateways presence.
  # On registration requests, accept and send a gateways status presence update.

	my $sid = shift;
	my $presence = shift;
	my $from = $presence->GetFrom();
	my $to = $presence->GetTo();
	my ($number) = split(/@/, $to);
	my $type = $presence->GetType();
	my $status = $presence->GetStatus();
	log3 ("Got `$type' type presence from $from to $to.");

	if ($type eq 'subscribe') {
	  	if ( ($number !~ /^\+[0-9]{11,11}$/) && ($to ne SERVICE_NAME.'/registered') ) {
			log3("Invalid number `$number' got.");
			sendPresence($presence, $from, $to, 'unsubscribed');
			return;
		}
		sendPresence($presence, $from, $to, 'subscribed', );
		if ($to eq SERVICE_NAME.'/registered') {
			sendPresence($presence, $from, $to, 'available', );
		} else {
			sendGWNumPresences($number, $from);
		}

	} elsif (($type eq 'available') or ($type eq 'probe')) {
		if ( $number =~ /^\+[0-9]{11,11}$/ ) {
			sendGWNumPresences($number, $from);
		}
		if ($to eq SERVICE_NAME.'/registered') {
		  sendPresence($presence, $from, $to, 'available', );
		}
	} elsif ($type eq 'unsubscribe') {
		sendPresence($presence, $from, $to, 'unsubscribed');
	}
}

sub Stop {
  # Terminate the SMS component's current run.
  print "Exiting...\n";
  $Connection->Disconnect();
  exit(0);
}

sub log1 {
  # WARN
  my $msg = shift;
  return unless VERBOSE >= 1;
  print STDERR "WARN: $msg\n";
}

sub log2 {
  # INFO
  my $msg = shift;
  return unless VERBOSE >= 2;
  print "INFO: $msg\n";
}

sub log3 {
  # DBUG
  my $msg = shift;
  return unless VERBOSE >= 3;
  print "DBUG: $msg\n";
}
